/**
 * ITI 1220 Fall 2005, Assignment 6, Question 3
 * 
 * @author Alan Williams
 */ 

public class Vigenere
{
	/**
	 * Encode a message encoded with the Vigenere cipher using a specified key
	 * phrase.
	 * <p>
	 * This method takes a string and encodes it with the Vigenere cipher using
	 * the key phrase. To do this, each letter in the message string is matched
	 * with a key phrase letter. The key phrase letter represents a Caesar shift
	 * value.
	 * <p>
	 * The method calls Caesar.encode( ) for each character, using the
	 * corresponding shift.  Finally, the message is broken into 5 letter groups.
	 * 
	 * @param message
	 *            The string to be encoded
	 * @param key
	 *            A key phrase string.
	 * @return The encoded message
	 */
	
	public static String encode( String message, String key )
	{
		// DECLARE VARIABLES / DATA DICTIONARY
		
		String result; 				// RESULT: the encoded message in 5 letter words
		String encodedMessage; 		// INTERMEDIATE: the encoded message
		String compressedMessage;	// INTERMEDIATE: Message with all non-lower
									//               case characters removed.
		String compressedKey;		// INTERMEDIATE: Key with all non-lower case
									//               characters removed.
		int messageIndex;			// INTERMEDIATE: location of current
									//               character
		char messageChar;			// INTERMEDIATE: current character in
									//               message
		int keyIndex;				// INTERMEDIATE: index of matching character
									//               in key phrase
		char keyChar;				// INTERMEDIATE: key character to use for
									//               current message character
		int keyValue;				// INTERMEDIATE: amount of Caesar shift to
									//               use for current message character
		String encodeChar; 			// INTERMEDIATE: decoded single character
		
		// BODY OF ALGORITHM
		
		// Start with empty string as result.
		
		encodedMessage = "";
		
		// Remove all non-lower case characters from both the
		// message and the key phrase.
		
		compressedMessage = Vigenere.compress( message );
		compressedKey = Vigenere.compress( key );

		// For each character in message:
		
		messageIndex = 0;
		while ( messageIndex < compressedMessage.length( ) )
		{
			// Get next character.
			
			messageChar = compressedMessage.charAt( messageIndex );
			
			// Determine which key letter to use. Finding the current
			// character position MOD the key length will match up the
			// correct character.
			
			keyIndex = messageIndex % compressedKey.length( );
			
			// Find the key letter at the appropriate index.
			
			keyChar = compressedKey.charAt( keyIndex );
			
			// Find the key character's relative difference from 'a'. This will
			// be the amount of the Caesar shift to use for this letter.
			
			keyValue = keyChar - 'a';
			
			// Call the Caesar encode method to get the encoded character.  Note
			// that the method returns a String instead of a char, but there should
			// be only one character in the String.
			
			encodeChar = Caesar
					.encode( String.valueOf( messageChar ), keyValue );
			
			// Append encoded character to the encoded string.
			
			encodedMessage = encodedMessage + encodeChar;
			
			// Go to next character in message.
			
			messageIndex = messageIndex + 1;
		}
		
		// Break message into 5 character "words"
		
		result = Vigenere.fiveLetterWords(encodedMessage);
		
		
		// RETURN RESULT
		
		return result;
	}
	
	/**
	 * Decode a message encoded with the Vigenere cipher using a specified key
	 * phrase.
	 * <p>
	 * This method takes a string that is presumed to be a message encoded with
	 * a Vigenere cipher, and reverses the encoding using the key phrase. To do
	 * this, each letter in the message string is matched with a key phrase
	 * letter. The key phrase letter represents a Caesar shift value.
	 * <p>
	 * The method calls Caesar.decode( ) for each character, using the
	 * corresponding shift.
	 * 
	 * @param message
	 *            The string to be decoded
	 * @param key
	 *            A key phrase string.
	 * @return The decoded message
	 */
	public static String decode( String message, String key )
	{
		// DECLARE VARIABLES / DATA DICTIONARY
		
		String result; 				// RESULT: the decoded message
		String compressedMessage;	// INTERMEDIATE: Message with all non-lower
									//               case characters removed.
		String compressedKey;		// INTERMEDIATE: Key with all non-lower case
									//               characters removed.
		int messageIndex;			// INTERMEDIATE: location of current
									//               character
		char messageChar;			// INTERMEDIATE: current character in
									//               message
		int keyIndex;				// INTERMEDIATE: index of matching character
									//               in key phrase
		char keyChar;				// INTERMEDIATE: key character to use for
									//               current message character
		int keyValue;				// INTERMEDIATE: amount of Caesar shift to
									//               use for current message character
		String decodeChar;			// INTERMEDIATE: decoded single character
		
		// BODY OF ALGORITHM
		
		// Start with empty string as result.
		
		result = "";
		
		// Remove all non-lower case characters from both the
		// message and the key phrase.
		
		compressedMessage = Vigenere.compress( message );
		compressedKey = Vigenere.compress( key );
		
		// For each character in message:
		
		messageIndex = 0;
		while ( messageIndex < compressedMessage.length( ) )
		{
			// Get next character.
			
			messageChar = compressedMessage.charAt( messageIndex );
			
			// Determine which key letter to use. Finding the current
			// character position MOD the key length will match up the
			// correct character.
			
			keyIndex = messageIndex % compressedKey.length( );
			
			// Find the key letter at the appropriate index.
			
			keyChar = compressedKey.charAt( keyIndex );
			
			// Find the key character's relative difference from 'a'. This will
			// be the amount of the Caesar shift to use for this letter.
			
			keyValue = keyChar - 'a';
			
			// Call the Caesar decode method to get the decoded character.  Note
			// that the method returns a String instead of a char, but there should
			// be only one character in the String.
			
			decodeChar = Caesar
					.decode( String.valueOf( messageChar ), keyValue );
			
			// Append decoded character to the decoded string.
			
			result = result + decodeChar;
			
			// Go to next character in message.
			
			messageIndex = messageIndex + 1;
		}
		
		// RETURN RESULT
		
		return result;
	}
	
	/**
	 * Remove all non-lower case characters from a string.
	 * 
	 * @param text String to be compressed.
	 * @return String with all non-lower case characters removed.
	 */
	
	public static String compress( String text )
	{
		// DECLARE VARIABLES / DATA DICTIONARY
		
		String result;		// RESULT:  text with all non-lower case characters removed
		int index;			// INTERMEDIATE:  current position within 'text'	
		char messageChar;	// INTERMEDIATE:  current character within 'text'
		
		// BODY OF ALGORITHM
		
		result = "";
		index = 0;
		
		// For each character in message:
		
		while ( index < text.length( ) )
		{
			// Get current character.
			
			messageChar = text.charAt( index );
			
			// Check if character is a lower case letter.
			
			if ( messageChar >= 'a' && messageChar <= 'z' )
			{
				// Pass lower case letter through to result
				
				result = result + messageChar;
			}
			else
			{
				// Skip all other characters.
				; // do nothing
			}
			
			// Go to next character
			
			index = index + 1;
		}
		
		// RETURN RESULT
		
		return result;
	}
	
	
	/**
	 * Remove all non-lower case characters from a string, and then
	 * break the remaining text into 5 letter "words".
	 * <p>
	 * If the text is a multiple of 5 letters, no trailing blank is added.
	 * 
	 * @param text String to be formatted.
	 * @return String in groups of 5 letters
	 */
	
	public static String fiveLetterWords( String text )
	{
		// DECLARE VARIABLES / DATA DICTIONARY
		
		String result;		// RESULT:  text broken into 5 letter "words"
		int index;			// INTERMEDIATE:  current position within 'text'	
		char messageChar;	// INTERMEDIATE:  current character within 'text'
		int counter;		// INTERMEDIATE:  number of characters since last blank

		// BODY OF ALGORITHM
		
		result = "";
		index = 0;
		counter = 0;
		
		// For each character in text:
		
		while ( index < text.length( ) )
		{
			// Get current character
			
			messageChar = text.charAt( index );
			
			// Check for lower case character
			
			if ( messageChar >= 'a' && messageChar <= 'z' )
			{
				// Pass lower case character through to result.
				
				result = result + messageChar;
			}
			else
			{
				// Skip all non-lower case characters
				
				; // do nothing
			}
			
			// Go to next character
			
			index = index + 1;
			
			// See if we have covered five characters since last blank.
			// If so, and we are not at the end of the text, then add
			// a blank character.
			
			counter = ( counter + 1 ) % 5;
			if ( counter == 0 && index < text.length( ) )
			{
				// It is time to insert a blank.
				
				result = result + " ";
			}
			else
			{
				; // do nothing
			}
		}
		
		// RETURN RESULT
		
		return result;
	}
}
