import junit.framework.Assert;
import junit.framework.TestCase;
/**
 * 
 */

/**
 * @author Alan Williams
 *
 */
public class VigenereTest extends TestCase
{

	public static void main( String[] args )
	{
		junit.swingui.TestRunner.run( VigenereTest.class );
	}

	/*
	 * Test method for 'Vigenere.encode(String, String)'
	 */
	public void testVEncode1( )
	{
		String testMessage = "to be or not to be that is the question";
		String testKey = "relations";
		
		String expected = "ksmeh zbblk smemp ogajx sejcs flzsy";
		String actual = Vigenere.encode(testMessage, testKey);
		
		Assert.assertEquals( expected, actual );
	}

	public void testVEncode2( )
	{
		String testMessage = "to be or not to be; that is the question!";
		String testKey = "b";
		
		String expected = "upcfp sopuu pcfui bujtu ifrvf tujpo";
		String actual = Vigenere.encode(testMessage, testKey);
		
		Assert.assertEquals( expected, actual );
	}

	public void testVEncode3( )
	{
		String testMessage = "to be or not to be that is the question";
		String testKey = "key with spaces";
		
		String expected = "dszaw kugit qfwdl ypqla ztqwi kdmmj";
		String actual = Vigenere.encode(testMessage, testKey);
		
		Assert.assertEquals( expected, actual );
	}

	/*
	 * Test method for 'Vigenere.decode(String, String)'
	 */
	public void testVDecode1( )
	{
		String testMessage = "ksmeh zbblk smemp ogajx sejcs flzsy";
		String testKey = "relations";
		
		String expected = "tobeornottobethatisthequestion";
		String actual = Vigenere.decode(testMessage, testKey);
		
		Assert.assertEquals( expected, actual );
	}

	public void testVDecode2( )
	{
		String testMessage = "upcfp sopuu pcfui bujtu ifrvf tujpo";
		String testKey = "b";
		
		String expected = "tobeornottobethatisthequestion";
		String actual = Vigenere.decode(testMessage, testKey);
		
		Assert.assertEquals( expected, actual );
	}

	public void testVDecode3( )
	{
		String testMessage = "dszaw kugit qfwdl ypqla ztqwi kdmmj";
		String testKey = "key with spaces";
		
		String expected = "tobeornottobethatisthequestion";
		String actual = Vigenere.decode(testMessage, testKey);
		
		Assert.assertEquals( expected, actual );
	}

	/*
	 * Test method for 'Vigenere.compress(String)'
	 */
	public void testCompress1( )
	{
		String testData = "return to  rome";
		
		String expected = "returntorome";
		String actual = Vigenere.compress(testData);
		
		Assert.assertEquals( expected, actual );
	}

	/*
	 * Test method for 'Vigenere.compress(String)'
	 */
	public void testCompress2( )
	{
		String testData = "return,to.@#$rome";
		
		String expected = "returntorome";
		String actual = Vigenere.compress(testData);
		
		Assert.assertEquals( expected, actual );
	}

	/*
	 * Test method for 'Vigenere.fiveLetterWords(String)'
	 */
	public void test5LWNotMod5( )
	{
		String testData = "tobeornottobe";
		
		String expected = "tobeo rnott obe";
		String actual = Vigenere.fiveLetterWords(testData);
		Assert.assertEquals(expected, actual);
	}

	/*
	 * Test method for 'Vigenere.fiveLetterWords(String)'
	 */
	public void test5LWMod5( )
	{
		String testData = "tobeornottobexx";
		
		String expected = "tobeo rnott obexx";
		String actual = Vigenere.fiveLetterWords(testData);
		Assert.assertEquals(expected, actual);
	}
}
